"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BytecodeGenerator = void 0;
const instruction_1 = require("./bytecode-generator/instruction");
const context_1 = require("./bytecode-generator/context");
const statement_1 = require("./bytecode-generator/statement");
const greybel_core_1 = require("greybel-core");
const error_1 = require("./utils/error");
const parse = function (code) {
    try {
        const parser = new greybel_core_1.Parser(code);
        return parser.parseChunk();
    }
    catch (err) {
        if (err instanceof error_1.PrepareError) {
            this.context.handler.errorHandler.raise(err);
        }
        else {
            this.context.handler.errorHandler.raise(new error_1.PrepareError(err.message, {
                range: err.range,
                target: this.context.target.peek()
            }));
        }
    }
};
class BytecodeGenerator {
    constructor(options) {
        var _a;
        this.context = (_a = options.context) !== null && _a !== void 0 ? _a : new context_1.Context({
            target: options.target,
            handler: options.handler,
            debugMode: options.debugMode
        });
    }
    compile(code) {
        return __awaiter(this, void 0, void 0, function* () {
            const statementGenerator = new statement_1.BytecodeStatementGenerator(this.context, parse.bind(this));
            const node = parse.call(this, code);
            yield statementGenerator.process(node);
            const mod = this.context.module.peek();
            mod.pushCode({
                op: instruction_1.OpCode.HALT,
                source: mod.getSourceLocation(node)
            });
            return {
                code: mod.getCode(),
                imports: this.context.imports
            };
        });
    }
}
exports.BytecodeGenerator = BytecodeGenerator;
