"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Module = void 0;
const stack_1 = require("../utils/stack");
class Module {
    constructor(target) {
        this._target = target;
        this._context = new stack_1.Stack({
            code: [],
            jumpPoints: []
        });
    }
    get target() {
        return this._target;
    }
    get context() {
        return this._context;
    }
    isGlobalScope() {
        return this._context.length === 1;
    }
    pushContext() {
        this._context.push({
            code: [],
            jumpPoints: []
        });
    }
    popContext() {
        return this._context.pop();
    }
    getCurrentIp() {
        return this._context.peek().code.length - 1;
    }
    getCode() {
        return this._context.peek().code;
    }
    pushCode(item) {
        item.ip = this.getCurrentIp() + 1;
        this._context.peek().code.push(item);
    }
    getJumpPoint() {
        const jumpPoints = this._context.peek().jumpPoints;
        if (jumpPoints.length === 0) {
            return null;
        }
        return jumpPoints[jumpPoints.length - 1];
    }
    pushJumppoint(start, end) {
        this._context.peek().jumpPoints.push([start, end]);
    }
    popJumppoint() {
        return this._context.peek().jumpPoints.pop();
    }
    getSourceLocation(node, name) {
        const target = this.target;
        return {
            name: name !== null && name !== void 0 ? name : node.type,
            path: target,
            start: node.start,
            end: node.end
        };
    }
    getInternalLocation() {
        return {
            name: 'internal',
            path: 'internal',
            start: { line: 0, character: 0 },
            end: { line: 0, character: 0 }
        };
    }
}
exports.Module = Module;
