import { ASTFeatureImportExpression, ASTFeatureIncludeExpression } from 'greybel-core';
import { ASTAssignmentStatement, ASTBase, ASTCallExpression, ASTEvaluationExpression, ASTForGenericStatement, ASTIdentifier, ASTIfStatement, ASTIndexExpression, ASTListConstructorExpression, ASTMapConstructorExpression, ASTMemberExpression, ASTReturnStatement, ASTUnaryExpression, ASTWhileStatement } from 'miniscript-core';
import { Context } from './context';
import { LineCallableContext, LineIdentifierContext } from './line';
import { IBytecodeExpressionGenerator, IBytecodeStatementGenerator, ParseCodeFunction } from './models';
export declare class BytecodeStatementGenerator implements IBytecodeStatementGenerator {
    protected context: Context;
    protected exprGenerator: IBytecodeExpressionGenerator;
    protected parseCode: ParseCodeFunction;
    constructor(context: Context, parseCodeFunction: ParseCodeFunction);
    process(node: ASTBase): Promise<void>;
    processMemberExpression(node: ASTMemberExpression, context?: LineCallableContext): Promise<void>;
    processIndexExpression(node: ASTIndexExpression, context?: LineCallableContext): Promise<void>;
    processIdentifier(node: ASTIdentifier, context?: LineIdentifierContext): Promise<void>;
    processAssignmentStatement(node: ASTAssignmentStatement): Promise<void>;
    processEvaluationExpression(node: ASTEvaluationExpression): Promise<void>;
    processReturn(node: ASTReturnStatement): Promise<void>;
    processBreak(node: ASTBase): Promise<void>;
    processContinue(node: ASTBase): Promise<void>;
    processMapConstructorExpression(node: ASTMapConstructorExpression): Promise<void>;
    processListConstructorExpression(node: ASTListConstructorExpression): Promise<void>;
    processWhileStatement(node: ASTWhileStatement): Promise<void>;
    processUnaryExpression(node: ASTUnaryExpression): Promise<void>;
    processCallExpression(node: ASTCallExpression): Promise<void>;
    processIfStatement(node: ASTIfStatement): Promise<void>;
    processForGenericStatement(node: ASTForGenericStatement): Promise<void>;
    protected createImport(node: ASTFeatureImportExpression, path: string, code: string): Promise<void>;
    processImportExpression(node: ASTFeatureImportExpression): Promise<void>;
    processIncludeExpression(node: ASTFeatureIncludeExpression): Promise<void>;
    processDebuggerExpression(node: ASTBase): Promise<void>;
}
