"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OperationContext = exports.Scope = exports.ContextType = void 0;
const map_1 = require("./types/map");
const string_1 = require("./types/string");
const stack_1 = require("./utils/stack");
var ContextType;
(function (ContextType) {
    ContextType[ContextType["Api"] = 0] = "Api";
    ContextType[ContextType["Global"] = 1] = "Global";
    ContextType[ContextType["Function"] = 2] = "Function";
    ContextType[ContextType["Injected"] = 3] = "Injected";
})(ContextType = exports.ContextType || (exports.ContextType = {}));
class Scope extends map_1.CustomMap {
    constructor(context) {
        super();
        this.context = context;
    }
    get(current, typeIntrinsics) {
        var _a, _b, _c, _d;
        if (this.has(current)) {
            return super.get(current, typeIntrinsics);
        }
        else if ((_a = this.context.outer) === null || _a === void 0 ? void 0 : _a.scope.has(current)) {
            return this.context.outer.scope.get(current, typeIntrinsics);
        }
        else if ((_b = this.context.globals) === null || _b === void 0 ? void 0 : _b.scope.has(current)) {
            return this.context.globals.scope.get(current, typeIntrinsics);
        }
        else if ((_c = this.context.api) === null || _c === void 0 ? void 0 : _c.scope.has(current)) {
            return this.context.api.scope.get(current, typeIntrinsics);
        }
        const intrinsics = (_d = typeIntrinsics.map) !== null && _d !== void 0 ? _d : map_1.CustomMap.getIntrinsics();
        if (intrinsics.has(current)) {
            return intrinsics.get(current);
        }
        throw new Error(`Unknown path ${current.toString()}.`);
    }
    getWithOrigin(current, typeIntrinsics) {
        var _a, _b, _c, _d;
        if (this.has(current)) {
            return super.getWithOrigin(current, typeIntrinsics);
        }
        else if ((_a = this.context.outer) === null || _a === void 0 ? void 0 : _a.scope.has(current)) {
            return this.context.outer.scope.getWithOrigin(current, typeIntrinsics);
        }
        else if ((_b = this.context.globals) === null || _b === void 0 ? void 0 : _b.scope.has(current)) {
            return this.context.globals.scope.getWithOrigin(current, typeIntrinsics);
        }
        else if ((_c = this.context.api) === null || _c === void 0 ? void 0 : _c.scope.has(current)) {
            return this.context.api.scope.getWithOrigin(current, typeIntrinsics);
        }
        const intrinsics = (_d = typeIntrinsics.map) !== null && _d !== void 0 ? _d : map_1.CustomMap.getIntrinsics();
        if (intrinsics.has(current)) {
            return {
                value: intrinsics.get(current),
                origin: null
            };
        }
        throw new Error(`Unknown path ${current.toString()}.`);
    }
}
exports.Scope = Scope;
class OperationContext {
    constructor(options) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
        this.iterators = new stack_1.Stack();
        this.code = options.code;
        this.cp = null;
        this.ip = 0;
        this.previous = (_a = options.previous) !== null && _a !== void 0 ? _a : null;
        this.type = (_b = options.type) !== null && _b !== void 0 ? _b : ContextType.Api;
        this.scope = this.type === ContextType.Injected ? options.previous.scope : new Scope(this);
        this.self = (_c = options.self) !== null && _c !== void 0 ? _c : null;
        this.super = (_d = options.super) !== null && _d !== void 0 ? _d : null;
        this.isProtected = (_e = options.isProtected) !== null && _e !== void 0 ? _e : false;
        this.isCalledByCommand = (_f = options.isCalledByCommand) !== null && _f !== void 0 ? _f : false;
        this.api = (_h = (_g = this.previous) === null || _g === void 0 ? void 0 : _g.api) !== null && _h !== void 0 ? _h : this.lookupApi();
        this.globals = (_k = (_j = this.previous) === null || _j === void 0 ? void 0 : _j.globals) !== null && _k !== void 0 ? _k : this.lookupGlobals();
        this.locals = (_l = this.lookupLocals()) !== null && _l !== void 0 ? _l : this;
        this.outer = (_m = options.outer) !== null && _m !== void 0 ? _m : this.lookupOuter();
    }
    lookupAllOfType(validate) {
        const me = this;
        const result = [];
        if (validate(me.type)) {
            result.push(me);
        }
        let current = me.previous;
        while (current) {
            if (validate(current.type)) {
                result.push(current);
            }
            current = current.previous;
        }
        return result;
    }
    lookupType(allowedTypes) {
        if (allowedTypes.has(this.type)) {
            return this;
        }
        let current = this.previous;
        while (current !== null) {
            if (allowedTypes.has(current.type)) {
                return current;
            }
            current = current.previous;
        }
        return null;
    }
    lookupAllScopes() {
        return this.lookupAllOfType((type) => [ContextType.Global, ContextType.Function].includes(type));
    }
    lookupApi() {
        return this.lookupType(OperationContext.lookupApiType);
    }
    lookupGlobals() {
        return this.lookupType(OperationContext.lookupGlobalsType);
    }
    lookupLocals() {
        return this.lookupType(OperationContext.lookupLocalsType);
    }
    lookupOuter() {
        var _a, _b;
        return (_b = (_a = this.locals.previous) === null || _a === void 0 ? void 0 : _a.lookupLocals()) !== null && _b !== void 0 ? _b : null;
    }
    extend(map) {
        this.scope.extend(map);
        return this;
    }
    set(path, value) {
        this.locals.scope.set(path, value);
    }
    get(path, contextTypeIntrinsics) {
        return this.locals.scope.get(path, contextTypeIntrinsics);
    }
    injectContext() {
        if (this.self)
            this.set(string_1.Self, this.self);
        if (this.super)
            this.set(string_1.Super, this.super);
    }
    getCurrentInstruction() {
        if (this.cp === null)
            return null;
        return this.code[this.cp];
    }
    fork(options) {
        return new OperationContext({
            previous: this,
            type: options.type,
            isProtected: false,
            code: options.code,
            self: options.self,
            super: options.super,
            outer: options.outer,
            isCalledByCommand: options.isCalledByCommand
        });
    }
}
OperationContext.lookupApiType = new Set([ContextType.Api]);
OperationContext.lookupGlobalsType = new Set([
    ContextType.Global
]);
OperationContext.lookupLocalsType = new Set([
    ContextType.Global,
    ContextType.Function
]);
exports.OperationContext = OperationContext;
