"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectValue = void 0;
const value_hash_1 = require("./value-hash");
class ObjectValue {
    constructor(entries) {
        if (entries == null) {
            this.data = new Map();
        }
        else if (entries instanceof ObjectValue) {
            this.data = new Map(entries.data);
        }
        else if (Array.isArray(entries)) {
            this.data = new Map();
            for (const [key, value] of entries) {
                this.set(key, value);
            }
        }
        if (this.data == null) {
            throw new Error('Unknown entries type.');
        }
    }
    get(mapKey) {
        const hash = (0, value_hash_1.valueHash)(mapKey);
        const keyPair = this.data.get(hash);
        if (!keyPair)
            return null;
        return keyPair[1];
    }
    has(mapKey) {
        const hash = (0, value_hash_1.valueHash)(mapKey);
        return this.data.has(hash);
    }
    set(mapKey, mapValue) {
        const hash = (0, value_hash_1.valueHash)(mapKey);
        this.data.set(hash, [mapKey, mapValue]);
        return this;
    }
    delete(mapKey) {
        const hash = (0, value_hash_1.valueHash)(mapKey);
        return this.data.delete(hash);
    }
    values() {
        return [...this.data.values()].map(([_, v]) => v);
    }
    keys() {
        return [...this.data.values()].map(([k]) => k);
    }
    entries() {
        return [...this.data.values()];
    }
    get size() {
        return this.data.size;
    }
    forEach(callback) {
        for (const [key, value] of this.data.values()) {
            callback(value, key, this);
        }
    }
    fork() {
        const newObject = new ObjectValue();
        for (const [key, value] of this.entries()) {
            newObject.set(key.fork(), value.fork());
        }
        return newObject;
    }
    extend(objVal) {
        for (const [key, value] of objVal.entries()) {
            this.set(key, value);
        }
        return this;
    }
    clear() {
        this.data.clear();
    }
}
exports.ObjectValue = ObjectValue;
